@@ -191,8 +191,8 @@ def cacheable(fn: T) -> T:
     return cached_strategy
 
 
-def base_defines_strategy(
-    force_reusable: bool, *, try_non_lazy: bool = False
+def defines_strategy(
+    *, force_reusable_values: bool = False, try_non_lazy: bool = False
 ) -> Callable[[T], T]:
     """Returns a decorator for strategy functions.
 
@@ -223,7 +223,7 @@ def base_defines_strategy(
                     # wrap that up in a LazyStrategy so it happens again later.
                     pass
             result = LazyStrategy(strategy_definition, args, kwargs)
-            if force_reusable:
+            if force_reusable_values:
                 result.force_has_reusable_values = True
                 assert result.has_reusable_values
             return result
@@ -234,11 +234,6 @@ def base_defines_strategy(
     return decorator
 
 
-defines_strategy = base_defines_strategy(False)
-defines_strategy_with_reusable_values = base_defines_strategy(True)
-defines_strategy_without_laziness = base_defines_strategy(False, try_non_lazy=True)
-
-
 class Nothing(SearchStrategy):
     def calc_is_empty(self, recur):
         return True
@@ -291,7 +286,7 @@ def just(value: T) -> SearchStrategy[T]:
     return JustStrategy(value)
 
 
-@defines_strategy_with_reusable_values
+@defines_strategy(force_reusable_values=True)
 def none() -> SearchStrategy[None]:
     """Return a strategy which only generates None.
 
@@ -303,12 +298,12 @@ def none() -> SearchStrategy[None]:
 
 @overload
 def one_of(args: Sequence[SearchStrategy[Any]]) -> SearchStrategy[Any]:
-    pass  # pragma: no cover
+    raise NotImplementedError
 
 
 @overload  # noqa: F811
 def one_of(*args: SearchStrategy[Any]) -> SearchStrategy[Any]:
-    pass  # pragma: no cover
+    raise NotImplementedError
 
 
 def one_of(*args):  # noqa: F811
@@ -348,7 +343,7 @@ def one_of(*args):  # noqa: F811
 
 
 @cacheable
-@defines_strategy_with_reusable_values
+@defines_strategy(force_reusable_values=True)
 def integers(min_value: int = None, max_value: int = None) -> SearchStrategy[int]:
     """Returns a strategy which generates integers.
 
@@ -405,7 +400,7 @@ def integers(min_value: int = None, max_value: int = None) -> SearchStrategy[int
 
 
 @cacheable
-@defines_strategy
+@defines_strategy()
 def booleans() -> SearchStrategy[bool]:
     """Returns a strategy which generates instances of :class:`python:bool`.
 
@@ -416,7 +411,7 @@ def booleans() -> SearchStrategy[bool]:
 
 
 @cacheable
-@defines_strategy_with_reusable_values
+@defines_strategy(force_reusable_values=True)
 @deprecated_posargs
 def floats(
     min_value: Real = None,
@@ -446,8 +441,6 @@ def floats(
     required to represent the generated float. Valid values are 16, 32, or 64.
     Passing ``width=32`` will still use the builtin 64-bit ``float`` class,
     but always for values which can be exactly represented as a 32-bit float.
-    Half-precision floats (``width=16``) are not supported on Python 3.5,
-    unless :pypi:`Numpy` is installed.
 
     The exclude_min and exclude_max argument can be used to generate numbers
     from open or half-open intervals, by excluding the respective endpoints.
@@ -478,10 +471,6 @@ def floats(
             "Got width=%r, but the only valid values are the integers 16, "
             "32, and 64." % (width,)
         )
-    if width == 16 and sys.version_info[:2] < (3, 6) and "numpy" not in sys.modules:
-        raise InvalidArgument(  # pragma: no cover
-            "width=16 requires either Numpy, or Python >= 3.6"
-        )
 
     check_valid_bound(min_value, "min_value")
     check_valid_bound(max_value, "max_value")
@@ -634,7 +623,7 @@ def floats(
 
 
 @cacheable
-@defines_strategy
+@defines_strategy()
 def tuples(*args: SearchStrategy) -> SearchStrategy[tuple]:
     """Return a strategy which generates a tuple of the same length as args by
     generating the value at index i from args[i].
@@ -652,16 +641,16 @@ def tuples(*args: SearchStrategy) -> SearchStrategy[tuple]:
 
 @overload
 def sampled_from(elements: Sequence[T]) -> SearchStrategy[T]:
-    pass  # pragma: no cover
+    raise NotImplementedError
 
 
 @overload  # noqa: F811
 def sampled_from(elements: Type[enum.Enum]) -> SearchStrategy[Any]:
     # `SearchStrategy[Enum]` is unreliable due to metaclass issues.
-    pass  # pragma: no cover
+    raise NotImplementedError
 
 
-@defines_strategy_without_laziness  # noqa: F811
+@defines_strategy(try_non_lazy=True)  # noqa: F811
 def sampled_from(elements):
     """Returns a strategy which generates any value present in ``elements``.
 
@@ -686,7 +675,7 @@ def sampled_from(elements):
         repr_ = "sampled_from(%s.%s)" % (elements.__module__, elements.__name__)
     else:
         repr_ = "sampled_from(%r)" % (elements,)
-    if hasattr(enum, "Flag") and isclass(elements) and issubclass(elements, enum.Flag):
+    if isclass(elements) and issubclass(elements, enum.Flag):
         # Combinations of enum.Flag members are also members.  We generate
         # these dynamically, because static allocation takes O(2^n) memory.
         # LazyStrategy is used for the ease of force_repr.
@@ -698,7 +687,7 @@ def sampled_from(elements):
 
 
 @cacheable
-@defines_strategy
+@defines_strategy()
 @deprecated_posargs
 def lists(
     elements: SearchStrategy[Ex],
@@ -789,7 +778,7 @@ def lists(
 
 
 @cacheable
-@defines_strategy
+@defines_strategy()
 @deprecated_posargs
 def sets(
     elements: SearchStrategy[Ex], *, min_size: int = 0, max_size: int = None
@@ -809,7 +798,7 @@ def sets(
 
 
 @cacheable
-@defines_strategy
+@defines_strategy()
 @deprecated_posargs
 def frozensets(
     elements: SearchStrategy[Ex], *, min_size: int = 0, max_size: int = None
@@ -836,7 +825,7 @@ class PrettyIter:
         return "iter({!r})".format(self._values)
 
 
-@defines_strategy
+@defines_strategy()
 @deprecated_posargs
 def iterables(
     elements: SearchStrategy[Ex],
@@ -862,7 +851,7 @@ def iterables(
     ).map(PrettyIter)
 
 
-@defines_strategy
+@defines_strategy()
 def fixed_dictionaries(
     mapping: Dict[T, SearchStrategy[Ex]],
     *,
@@ -904,7 +893,7 @@ def fixed_dictionaries(
 
 
 @cacheable
-@defines_strategy
+@defines_strategy()
 @deprecated_posargs
 def dictionaries(
     keys: SearchStrategy[Ex],
@@ -940,7 +929,7 @@ def dictionaries(
 
 
 @cacheable
-@defines_strategy_with_reusable_values
+@defines_strategy(force_reusable_values=True)
 @deprecated_posargs
 def characters(
     *,
@@ -1047,7 +1036,7 @@ def characters(
 
 
 @cacheable
-@defines_strategy_with_reusable_values
+@defines_strategy(force_reusable_values=True)
 @deprecated_posargs
 def text(
     alphabet: Union[Sequence[str], SearchStrategy[str]] = characters(
@@ -1103,7 +1092,7 @@ def text(
 
 
 @cacheable
-@defines_strategy
+@defines_strategy()
 @deprecated_posargs
 def from_regex(
     regex: Union[AnyStr, Pattern[AnyStr]], *, fullmatch: bool = False
@@ -1144,7 +1133,7 @@ def from_regex(
 
 
 @cacheable
-@defines_strategy_with_reusable_values
+@defines_strategy(force_reusable_values=True)
 @deprecated_posargs
 def binary(*, min_size: int = 0, max_size: int = None) -> SearchStrategy[bytes]:
     """Generates :class:`python:bytes`.
@@ -1164,7 +1153,7 @@ def binary(*, min_size: int = 0, max_size: int = None) -> SearchStrategy[bytes]:
 
 
 @cacheable
-@defines_strategy
+@defines_strategy()
 def randoms(
     *, note_method_calls: bool = False, use_true_random: bool = None
 ) -> SearchStrategy[random.Random]:
@@ -1217,7 +1206,7 @@ class RandomModule(SearchStrategy):
 
 
 @cacheable
-@defines_strategy
+@defines_strategy()
 def random_module() -> SearchStrategy[RandomSeeder]:
     """The Hypothesis engine handles PRNG state for the stdlib and Numpy random
     modules internally, always seeding them to zero and restoring the previous
@@ -1240,7 +1229,7 @@ def random_module() -> SearchStrategy[RandomSeeder]:
 # Note that for the benefit of documentation and introspection tools, we set the
 # __signature__ attribute to show the semantic rather than actual signature.
 @cacheable
-@defines_strategy
+@defines_strategy()
 def builds(
     *callable_and_args: Union[Callable[..., Ex], SearchStrategy[Any]],
     **kwargs: Union[SearchStrategy[Any], InferType]
@@ -1452,13 +1441,6 @@ def _from_type(thing: Type[Ex]) -> SearchStrategy[Ex]:
         if getattr(thing, "__origin__", None) is typing.Union:
             args = sorted(thing.__args__, key=types.type_sorting_key)
             return one_of([from_type(t) for t in args])
-    # We can't resolve forward references, and under Python 3.5 (only)
-    # a forward reference is an instance of type.  Hence, explicit check:
-    elif type(thing) == getattr(typing, "_ForwardRef", None):  # pragma: no cover
-        raise ResolutionFailed(
-            "thing=%s cannot be resolved.  Upgrading to python>=3.6 may "
-            "fix this problem via improvements to the typing module." % (thing,)
-        )
     if not types.is_a_type(thing):
         raise InvalidArgument("thing=%s must be a type" % (thing,))
     # Now that we know `thing` is a type, the first step is to check for an
@@ -1524,7 +1506,7 @@ def _from_type(thing: Type[Ex]) -> SearchStrategy[Ex]:
 
 
 @cacheable
-@defines_strategy_with_reusable_values
+@defines_strategy(force_reusable_values=True)
 @deprecated_posargs
 def fractions(
     min_value: Union[Real, str] = None,
@@ -1611,7 +1593,7 @@ def fractions(
 
 
 def _as_finite_decimal(
-    value: Union[Real, str, None], name: str, allow_infinity: Optional[bool],
+    value: Union[Real, str, None], name: str, allow_infinity: Optional[bool]
 ) -> Optional[Decimal]:
     """Convert decimal bounds to decimals, carefully."""
     assert name in ("min_value", "max_value")
@@ -1634,7 +1616,7 @@ def _as_finite_decimal(
 
 
 @cacheable
-@defines_strategy_with_reusable_values
+@defines_strategy(force_reusable_values=True)
 @deprecated_posargs
 def decimals(
     min_value: Union[Real, str] = None,
@@ -1770,7 +1752,7 @@ class PermutationStrategy(SearchStrategy):
         return result
 
 
-@defines_strategy
+@defines_strategy()
 def permutations(values: Sequence[T]) -> SearchStrategy[List[T]]:
     """Return a strategy which returns permutations of the ordered collection
     ``values``.
@@ -1827,7 +1809,7 @@ def composite(f: Callable[..., Ex]) -> Callable[..., SearchStrategy[Ex]]:
     }
     new_argspec = argspec._replace(args=argspec.args[1:], annotations=annots)
 
-    @defines_strategy
+    @defines_strategy()
     @define_function_signature(f.__name__, f.__doc__, new_argspec)
     def accept(*args, **kwargs):
         return CompositeStrategy(f, args, kwargs)
@@ -1836,7 +1818,7 @@ def composite(f: Callable[..., Ex]) -> Callable[..., SearchStrategy[Ex]]:
     return accept
 
 
-@defines_strategy_with_reusable_values
+@defines_strategy(force_reusable_values=True)
 @cacheable
 @deprecated_posargs
 def complex_numbers(
@@ -1946,7 +1928,7 @@ def shared(base: SearchStrategy[Ex], *, key: Hashable = None) -> SearchStrategy[
 
 
 @cacheable
-@defines_strategy_with_reusable_values
+@defines_strategy(force_reusable_values=True)
 @deprecated_posargs
 def uuids(*, version: int = None) -> SearchStrategy[UUID]:
     """Returns a strategy that generates :class:`UUIDs <uuid.UUID>`.
@@ -1991,7 +1973,7 @@ class RunnerStrategy(SearchStrategy):
             return runner
 
 
-@defines_strategy_with_reusable_values
+@defines_strategy(force_reusable_values=True)
 @deprecated_posargs
 def runner(*, default: Any = not_set) -> SearchStrategy[Any]:
     """A strategy for getting "the current test runner", whatever that may be.
@@ -2173,7 +2155,7 @@ def deferred(definition: Callable[[], SearchStrategy[Ex]]) -> SearchStrategy[Ex]
     return DeferredStrategy(definition)
 
 
-@defines_strategy_with_reusable_values
+@defines_strategy(force_reusable_values=True)
 def emails() -> SearchStrategy[str]:
     """A strategy for generating email addresses as unicode strings. The
     address format is specified in :rfc:`5322#section-3.4.1`. Values shrink
@@ -2192,7 +2174,7 @@ def emails() -> SearchStrategy[str]:
     )
 
 
-@defines_strategy
+@defines_strategy()
 @deprecated_posargs
 def functions(
     *,
